#!/usr/perl/bin

=head1 NAME

perltobin - turn your perl script into a binary executable

=head1 DESCRIPTION

Do you have a perl script you'd like to distribute in executable form?

Then PerlBin is for you, whatever your OS may be, as long as there is a perl
port for it (you can run perl on it).

=head1 SYNOPSIS

    PerlBin [-?] [-o outdir] [-a mod] Script.pl

=head1 OPTIONS

=head2 [-?]

help! help! help! to see more help, try --man.

=head2 [-a mod]

Add a module to be scanned (must be in C<@INC>).
You can specify B<multiple> by doing C<-a foo -a bar -a gahZbo>.

=head2 -o outdir

A directory where to I<place> all the I<stuff>.
Defaults to C<Script_perlbined>.

=cut

use Config;

use Getopt::Long;
use Pod::Usage();

use PerlBin;
use File::Basename;
use File::Spec::Functions qw[ catfile rel2abs updir ];

use strict;
BEGIN{ eval q[use warnings;]; $^W++ if $@; }

use vars qw[$VERSION];
$VERSION = 0.01;

my %Opts;

GetOptions(\%Opts, qw[ help|?! gui! man! outdir|o=s ], 'a=s' => $Opts{a} = []);

sub pod2usage {
## make pod2usage work if perltobin.pl is packaged with perltobin.pl ;D
    my $f = rel2abs(__FILE__);
    open(INF,$f) or die "crap $f $!";
    seek INF, $PerlBin::skipTo, 0;

    Pod::Usage::pod2usage(
        -exitval => 1,
#        -input => __FILE__,
        -input => \*INF,
        -output => \*STDOUT,
        -verbose => shift,
        -message => shift || '',
    );
}

pod2usage(2) if $Opts{man};
pod2usage(1) if $Opts{help};


if( $Opts{gui} ){
    eval q[require PerlBin::Wx;];
    unless($@){
        PerlBin::Wx->new->MainLoop;
        exit(0);
    }
    eval q[use PerlBin::Tk;];
    unless($@){
        PerlBin::Tk->new->MainLoop;
        exit(0);
    }
    warn "You don't have Tk nor Wx installed, can't have GUI!!!\n\n";
}

pod2usage(0) if not @ARGV;

my $script  = $ARGV[0];
my $outdir  = $Opts{outdir} || basename($script, qw[ .pl .pm .t .cgi .fcgi ]).'_perlbined';

die "$script doesn't exist or is not a file" unless -e $script and not -d $script;

my $perlbin = catfile dirname(rel2abs __FILE__), 'PerlBin.dat';

$script = rel2abs $script;
push @INC, dirname $script; # cause of scan_deps

my $Depends = PerlBin->new($script);

$Depends->add_deps( @{$Opts{a}} )   if @{$Opts{a}};


my $outfile = catfile $outdir, basename($script, qw[ .pl .pm .t .cgi .fcgi ]);;

$outfile .= $Config{exe_ext};

$Depends->PutDeps($outdir);
$Depends->PutSO($outdir);
$Depends->PutBinary( $script => $outfile );



__END__

=head1 CAVEATS

None I can see. Beware of -s __FILE__
cause it will return the filesize of the exe, not your source.

The global C<$PerlBin::skipTo> holds the size of the binary
before your script was appended,
and you can use it as an offset.

pod2usage/perldoc don't like binary files for some reason.

See C<sub pod2usage> in here for a workaround.


=head1 SEE ALSO (RESOURCES)

All of the software listed here is
really useful, so check it out, before
you ask "why doesn't perlbin do x?" ;D

=head2 C<Module::CryptSource>

Since neither perlbin, nor PAR, nor App::Packer
even attempt to obscure the source code
Autrijus Tang developed this one for use with PAR
and others like it.

=head2 PAR 

Kinda like perlbin (same idea), except 
uses lots and lots of perl magic and requires perl 5.6.

L<http://search.cpan.org/author/AUTRIJUS/PAR/>

=head2 App::Packer

Kinda like perlbin (same idea), except packs
B<ALL> the file into this one, uses lots and lots of perl magic,
and requires perl 5.6.

L<http://search.cpan.org/author/MBARBON/App-Packer/>

=head2 C<exetype>

Change the nature of your win32 perlbin generated executable.
By default all are I<console> executables (dos window).

I distribute this one with perlbin.

=head2 NSIS

Nullsoft (SuperPiMP|Scriptable) Install System

http://www.nullsoft.com/free/nsis/ 

Win32 only.  For other os's you can use RPM ;) or family,
and there ain't never nothing wrong with a neat-o tarball ;D

=head2  UPX

http://upx.sf.net - you can pack individual .dll/.so's with this to save space (smaller sizes).

=head2 Pod::Stripper

Gets rid of pod in modules, cause you don't need it.
http://search.cpan.org/author/PODMASTER/Pod-Stripper/

Works pretty well but please use at your own risk.

And while i'm talking about pod,
I<Holy Getopt::Long, Pod::Usage Man!>
(http://perlmonks.org/index.pl?node_id=155288;)
cause  you B<gotta> have documentation.


=head2 Programmer's Tools

Lots of neat-o utilities and interesting reading here.

    http://www.programmerstools.org
    http://protools.owns.it
    http://protools.cjb.net

=head1 COPYRIGHT

Copyright (c) 2002-2003 by the http://perlbin.sf.net/ team.

Original code by Graciliano M. P.

Original idea not so original ;)

This here program and all enclosed *schtuff*
is distributed under the same terms as perl itself.
See perl.com for more info on that.

=cut
